@@ -2,7 +2,7 @@
  * comm.h - command name length definition 
  *
  * Copyright 1995 Werner Almesberger
- * Copyright 2012-2018 Craig Small <csmall@enc.com.au>
+ * Copyright 2012-2019 Craig Small <csmall@dropbear.xyz>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,4 +33,10 @@
  */
 #define COMM_LEN 64
 
+/*
+ * Older kernels had only 16 characters, which means we may have to check this
+ * too
+ */
+#define OLD_COMM_LEN 16
+
 #endif
