@@ -4,7 +4,7 @@
  * Based on fuser.c Copyright (C) 1993-2005 Werner Almesberger and Craig Small
  *
  * Completely re-written
- * Copyright (C) 2005-2017 Craig Small
+ * Copyright (C) 2005-2019 Craig Small <csmall@dropbear.xyz>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -188,6 +188,9 @@ scan_procs(struct names *names_head, struct inode_list *ino_head,
 	pid_t pid, my_pid;
 	uid_t uid;
 
+	if ( (ino_head == NULL) && (dev_head == NULL) )
+		return;
+
 	if ((topproc_dir = opendir("/proc")) == NULL) {
 		fprintf(stderr, _("Cannot open /proc directory: %s\n"),
 			strerror(errno));
@@ -700,7 +703,8 @@ find_net_sockets(struct inode_list **ino_list,
 	FILE *fp;
 	char pathname[200], line[BUFSIZ];
 	unsigned long loc_port, rmt_port;
-	unsigned long rmt_addr, scanned_inode;
+	unsigned long rmt_addr;
+    unsigned long long scanned_inode;
 	ino_t inode;
 	struct ip_connections *conn_tmp;
 
@@ -715,7 +719,7 @@ find_net_sockets(struct inode_list **ino_list,
 	while (fgets(line, BUFSIZ, fp) != NULL) {
 		if (sscanf
 		    (line,
-		     "%*u: %*x:%lx %08lx:%lx %*x %*x:%*x %*x:%*x %*x %*d %*d %lu",
+		     "%*u: %*x:%lx %08lx:%lx %*x %*x:%*x %*x:%*x %*x %*d %*d %llu",
 		     &loc_port, &rmt_addr, &rmt_port, &scanned_inode) != 4)
 			continue;
 #ifdef DEBUG
@@ -766,7 +770,7 @@ find_net6_sockets(struct inode_list **ino_list,
 	unsigned int tmp_addr[4];
 	char rmt_addr6str[INET6_ADDRSTRLEN];
 	struct ip6_connections *conn_tmp;
-	unsigned long scanned_inode;
+	unsigned long long scanned_inode;
 	ino_t inode;
 
 	if (snprintf(pathname, 200, "/proc/net/%s6", protocol) < 0)
@@ -782,7 +786,7 @@ find_net6_sockets(struct inode_list **ino_list,
 	while (fgets(line, BUFSIZ, fp) != NULL) {
 		if (sscanf
 		    (line,
-		     "%*u: %*x:%lx %08x%08x%08x%08x:%lx %*x %*x:%*x %*x:%*x %*x %*d %*d %lu",
+		     "%*u: %*x:%lx %08x%08x%08x%08x:%lx %*x %*x:%*x %*x:%*x %*x %*d %*d %llu",
 		     &loc_port, &(tmp_addr[0]), &(tmp_addr[1]), &(tmp_addr[2]),
 		     &(tmp_addr[3]), &rmt_port, &scanned_inode) != 7)
 			continue;
@@ -1699,7 +1703,7 @@ void fill_unix_cache(struct unixsocket_list **unixsocket_head)
 {
 	FILE *fp;
 	char line[BUFSIZ];
-	int scanned_inode;
+	unsigned long long scanned_inode;
 	struct stat st;
 	struct unixsocket_list *newsocket;
 
@@ -1711,7 +1715,7 @@ void fill_unix_cache(struct unixsocket_list **unixsocket_head)
 	while (fgets(line, BUFSIZ, fp) != NULL) {
 		char *path;
 		char *scanned_path = NULL;
-		if (sscanf(line, "%*x: %*x %*x %*x %*x %*d %d %ms",
+		if (sscanf(line, "%*x: %*x %*x %*x %*x %*d %llu %ms",
 			   &scanned_inode, &scanned_path) != 2) {
 			if (scanned_path)
 				free(scanned_path);
@@ -1873,6 +1877,10 @@ scan_knfsd(struct names *names_head, struct inode_list *ino_head,
 	char *find_space;
 	struct stat st;
 
+	if ( (ino_head == NULL) && (dev_head == NULL) )
+		return;
+
+
 	if ((fp = fopen(KNFSD_EXPORTS, "r")) == NULL) {
 #ifdef DEBUG
 		printf("Cannot open %s\n", KNFSD_EXPORTS);
@@ -1919,6 +1927,10 @@ scan_mounts(struct names *names_head, struct inode_list *ino_head,
 	char *find_space;
 	struct stat st;
 
+	if ( (ino_head == NULL) && (dev_head == NULL) )
+		return;
+
+
 	if ((fp = fopen(PROC_MOUNTS, "r")) == NULL) {
 		fprintf(stderr, "Cannot open %s\n", PROC_MOUNTS);
 		return;
@@ -1962,6 +1974,9 @@ scan_swaps(struct names *names_head, struct inode_list *ino_head,
 	char *find_space;
 	struct stat st;
 
+	if ( (ino_head == NULL) && (dev_head == NULL) )
+		return;
+
 	if ((fp = fopen(PROC_SWAPS, "r")) == NULL) {
 		/*fprintf(stderr, "Cannot open %s\n", PROC_SWAPS); */
 		return;
